{
Description :
This unit implement a graph class (technically a DiGraph see the TODO section)
It also provides some classical algo's (a very small part) ...
Waiting for contributors and feedbacks ;)

Author :
Michel Derie (derie@cragx.fgov.be)

Distribution :
SourceWare, FunWare, FreeWare, SmileWare, MirrWare, ArmoirWare ;-), ... etc

Remarks & improvements :
On the design, the extensions or anything else ... ARE welcome !
This unit does NOT pretend to be an efficient implementation
of graph concepts !!!

Legal disclaimer :
No warrenty of any kind ! Not fully tested so bugs can be present ;(
see the TODO section

Compiler :
Delphi 5 SP 1 ... Since it is a base classe it should compile on Delphi 3 & 4 also (I guess)

History :
Date        Author   Version   Comment
12/2/2001   MDE      v0.1      Creation & release !-) Thanks to torry's site
}
unit GraphClass;

interface

uses
  Classes, SysUtils, Contnrs ; // Contnrs is a very interesting and not enough famous unit, check it !

type
  { Forward declarations !!! }
  TNode = class ;
  TGraph = class ;

  TNodeEvent = procedure (Node : TNode) of object ;
  TNodeEventProc = procedure (ptr : pointer ; Node : TNode) ; // See CastToNodeEvent function !
  EGraphException = class (Exception) ;

  { We have a common ancestor for the nodes and the edges. It holds
    an integer value, a state and a generic pointer }
  TGraphItem = class
  public
    Data : pointer ;
    Value : integer ;
    State : integer ;
  end ;

  { Edge = arc = line = link = etc ... }
  TEdge = class (TGraphItem)
  private
    FPrevNode : TNode ;
    FNextNode : TNode ;
  public
    constructor Create (PrevNode, NextNode : TNode) ; virtual ;
    property PrevNode : TNode read FPrevNode ;
    property NextNode : TNode read FNextNode ;
  end ;

  { Node = vertex (plural vertices) = dot = point = etc ...  }
  TNode = class (TGraphItem)
  private
    FGraph : TGraph ;
    FNextEdges : TObjectList ;
    FPrevEdges : TObjectList ;
    function GetPrevEdge (iIndex : integer) : TEdge ;
    function GetPrevEdgeCount : integer ;
    function GetNextEdge (iIndex : integer) : TEdge ;
    function GetNextEdgeCount : integer ;
    procedure AddEdge (EdgesList : TObjectList ; Edge: TEdge) ;
    procedure DelEdge (EdgesList : TObjectList ; Edge : TEdge) ;
    function GetConnected : boolean ;
  public
    constructor Create ; virtual ;
    destructor Destroy ; override ;
    procedure AddNextEdge (Edge : TEdge) ;
    procedure AddPrevEdge (Edge : TEdge) ;
    procedure DelPrevEdge (Edge : TEdge) ;
    procedure DelNextEdge (Edge : TEdge) ;
    function GetPrevEdgeIndex (Edge : TEdge) : integer ;
    function GetNextEdgeIndex (Edge : TEdge) : integer ;
    property PrevEdges [iIndex : integer] : TEdge read GetPrevEdge ;
    property PrevEdgeCount : integer read GetPrevEdgeCount ;
    property NextEdges [iIndex : integer] : TEdge read GetNextEdge ;
    property NextEdgeCount : integer read GetNextEdgeCount ;
    property Graph : TGraph read FGraph ;
    property Connected : boolean read GetConnected ;
  end ;

  TGraph = class
  private
    FNodes : TObjectList ;
    function GetNode (iIndex : integer) : TNode ;
    function GetNodesCount : integer ;
  public
    constructor Create ; virtual ;
    destructor Destroy ; override ;
    procedure AddNode (Node : TNode) ;
    procedure DelNode (Node : TNode) ;
    function GetNodeIndex (Node : TNode) : integer ;
    property Nodes [iIndex : integer] : TNode read GetNode ;
    property NodeCount : integer read GetNodesCount ;
  end ;

const
  ST_NONE = 0 ;
  ST_PROCESSED = 1 ;

{ Helpers }
function CastToNodeEvent (proc : TNodeEventProc; ptr : pointer) : TNodeEvent ;
{$ifdef DEBUG}
procedure ShowEdge (Edge : TEdge) ;
procedure ShowNode (Node : TNode) ;
procedure ShowGraph (Graph : TGraph) ;
procedure ShowGraphEdges (Graph : TGraph) ;
procedure ShowGraphNodes (Graph : TGraph) ;
{$endif}
procedure ResetEdgesState (Graph : TGraph) ;
procedure ResetNodesState (Graph : TGraph) ;
procedure ResetGraph (Graph : TGraph) ;
function FindNodeByValue (Graph : TGraph ; iFromIndex, iValue : integer) : TNode ;
function FindNodeByData (Graph : TGraph ; iFromIndex : integer ; pData: pointer) : TNode ;
{ Walking graph algo's }
procedure TopologicalSort (Graph : TGraph ; OnProcessNode : TNodeEvent) ;
procedure DepthFirst (Node : TNode ; OnProcessNode : TNodeEvent) ;
procedure BreadthFirst (Node : TNode ; OnProcessNode : TNodeEvent) ;

{
TODO section :
- Add others classical graph routines
(i.e. Hamilton, Cycle detection, Prim, Dijkstra, etc ... )
- Verify that everything works fine in some extreme conditions
(no nodes, edges cycles on the same node, etc ...)
- Check to unify walking algo's by introducing iterator concept
- Verify if TGraph, TNode & TEdge are ready for derivation
- Check to introduce difference between Graph & DiGraph
(where arcs are directed) ... Infact here, we have a TDiGraph.
We should derive TGraph from TDiGraph or the opposite ?-)
- Add support for quicker nodes & edges additions(/suppressions ?)
- Add support to a TEdgeEvent ?
- Make a demo prog
- Think about add write capabilities to some properties
- ...
}

implementation

{$ifdef DEBUG}
uses
  Dialogs ;
{$endif}

///////////////////////////////////////////////////////////
{ Helper routines }
///////////////////////////////////////////////////////////

{
Description : little trick where ptr emulate a dummy self pointer
}
function CastToNodeEvent (proc : TNodeEventProc ; ptr : Pointer) : TNodeEvent ;
begin
	TMethod (result).data := ptr;
  TMethod (result).code := @proc;
end;

{$ifdef DEBUG}
procedure ShowEdge (Edge : TEdge) ;
begin
  ShowMessage ('Edge addresse = ' + IntToStr (integer (@Edge)) +
               ' Edge value = ' + IntToStr (Edge.Value) +
               ' Edge prev node value = ' + IntToStr (Edge.PrevNode.Value) +
               ' Edge next node value = ' + IntToStr (Edge.NextNode.Value)) ;
end ;

procedure ShowNode (Node : TNode) ;
begin
  ShowMessage ('Node addresse = ' + IntToStr (integer (@Node)) +
               ' Node value = ' + IntToStr (Node.Value) +
               ' Node #prev = ' + IntToStr (Node.PrevEdgeCount) +
               ' Node #next = ' + IntToStr (Node.NextEdgeCount)) ;
end ;

procedure ShowGraph (Graph : TGraph) ;
var
  i, j : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    ShowNode (Graph.Nodes [i]) ;
    { We can choose to naviguate using the Next XOR the Prev Edges ... }
    for j := 0 to Graph.Nodes [i].GetNextEdgeCount - 1 do begin
      ShowEdge (Graph.Nodes [i].NextEdges [j]) ;
    end ;
  end ;
end ;

procedure ShowGraphEdges (Graph : TGraph) ;
var
  i, j : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    { We can choose to naviguate using the Next XOR the Prev Edges ... }
    for j := 0 to Graph.Nodes [i].GetNextEdgeCount - 1 do begin
      ShowEdge (Graph.Nodes [i].NextEdges [j]) ;
    end ;
  end ;
end ;

procedure ShowGraphNodes (Graph : TGraph) ;
var
  i : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    ShowNode (Graph.Nodes [i]) ;
  end ;
end ;
{$endif}

procedure ResetEdgesState (Graph : TGraph) ;
var
  i, j : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    for j := 0 to Graph.Nodes [i].GetNextEdgeCount - 1 do begin
      Graph.Nodes [i].NextEdges [j].State := ST_NONE ;
    end ;
  end ;
end ;

procedure ResetNodesState (Graph : TGraph) ;
var
  i : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    Graph.Nodes [i].State := 0 ;
  end ;
end;

procedure ResetGraph (Graph : TGraph) ;
var
  i, j : integer ;

begin
  for i := 0 to Graph.NodeCount - 1 do begin
    Graph.Nodes [i].State := ST_NONE ;
    { We can choose to naviguate using the Next XOR the Prev Edges ... }
    for j := 0 to Graph.Nodes [i].GetNextEdgeCount - 1 do begin
      Graph.Nodes [i].NextEdges [j].State := ST_NONE ;
    end ;
  end ;
end;

function FindNodeByValue(Graph : TGraph ; iFromIndex, iValue: integer): TNode;
var
  i : integer ;

begin
  result := nil ;
  for i := iFromIndex to Graph.NodeCount - 1 do begin
    if Graph.Nodes [i].Value = iValue then begin
      result := Graph.Nodes [i] ;
      exit ;
    end ;
  end ;
end;

function FindNodeByData(Graph : TGraph ; iFromIndex : integer ; pData: pointer): TNode;
var
  i : integer ;

begin
  result := nil ;
  for i := iFromIndex to Graph.NodeCount - 1 do begin
    if Graph.Nodes [i].Data = pData then begin
      result := Graph.Nodes [i] ;
      exit ;
    end ;
  end ;
end ;

///////////////////////////////////////////////////////////
{ Walking graph algo's }
///////////////////////////////////////////////////////////

{ Warning : this method calls ResetGraph !!! }
procedure TopologicalSort(Graph : TGraph ; OnProcessNode: TNodeEvent);
var
  i, j, k : integer ;
  iNodeCount : integer ;
  iEdgeCount : integer ;
  bProcessNode : boolean ;
  bHasCycle : boolean ;
  Node : TNode ;

begin
  ResetGraph (Graph) ;
  i := 0 ;
  iNodeCount := Graph.NodeCount ;

  { While some nodes need to be processed }
  while i < iNodeCount do begin
    j := 0 ;
    bHasCycle := true ; // Avoid infinite loops
    { Search for unprocessed and prevless nodes }
    while j < iNodeCount do begin
      Node := Graph.Nodes [j] ;
      if Node.State <> ST_PROCESSED then begin
        k := 0 ;
        iEdgeCount := Node.PrevEdgeCount ;
        bProcessNode := true ;

        { Verify that the node have all its edges processed }
        while k < iEdgeCount do begin
          if Node.PrevEdges [k].State <> ST_PROCESSED then begin
            { We can't process it now }
            bProcessNode := false ;
            break ;
          end ;
          inc (k) ;
        end ;

        { We find one }
        if bProcessNode then begin
          if Assigned (OnProcessNode) then begin
            OnProcessNode (Node) ;
          end ;
          bHasCycle := false ;
          inc (i) ;
          Node.State := ST_PROCESSED ;
          iEdgeCount := Node.NextEdgeCount ;
          for k := 0 to iEdgeCount - 1 do begin
            Node.NextEdges [k].State := ST_PROCESSED ;
          end ;
        end ;
      end ; { if Nodes [j].State <> PROCESSED }
      inc (j) ;
    end ; { while j < iNodeCount }
    if bHasCycle then begin
      raise EGraphException.Create ('Graph can''t contains cycles for a topological sort') ;
    end ;
  end ; { while i < iNodeCount }
end;

{ Warning : this method calls ResetNodes !!! }
procedure GraphIterator (NodeList : TOrderedList ; Node : TNode ; OnProcessNode : TNodeEvent) ;
var
  i, j : integer ;
  iNodeCount : integer ;
  iEdgeCount : integer ;
  bProcessNode : boolean ;
  Graph : TGraph ;

begin
  Graph := Node.Graph ;
  ResetNodesState (Graph) ;
  iNodeCount := Graph.NodeCount ;
  NodeList.Push (Node) ;
  i := 0 ;

  while i < iNodeCount do begin
    { Is the stack empty ? }
    if NodeList.Count = 0 then begin
      { Yes let's find another starting node }
      j := 0 ;
      while j < iNodeCount do begin
        if Graph.Nodes [j].State <> ST_PROCESSED then begin
          NodeList.Push (Graph.Nodes [j]) ;
          break ;
        end ;
        inc (j) ;
      end ;
    end ;

    Node := TNode (NodeList.Peek) ;

    { Does the current node have unprocessed successors ? }
    iEdgeCount := Node.NextEdgeCount ;
    bProcessNode := true ;

    for j := 0 to iEdgeCount - 1 do begin
      if Node.NextEdges [j].NextNode.State <> ST_PROCESSED then begin
        NodeList.Push (Node.NextEdges [j].NextNode) ;

        if NodeList is TStack then begin
          { We use the class type of the NodeList container !!! }
          bProcessNode := false ;
        end ;
      end ;
    end ;

    { We find one }
    if bProcessNode then begin
      NodeList.Pop ;
      inc (i) ;
      Node.State := ST_PROCESSED ;
      if Assigned (OnProcessNode) then begin
        OnProcessNode (Node) ;
      end ;
    end ;
  end ; { while i < iNodeCount }
end ;

procedure DepthFirst (Node : TNode ; OnProcessNode : TNodeEvent) ;
var
  Stack : TStack ;

begin
  Stack := TStack.Create ;
  try
    GraphIterator (Stack,Node,OnProcessNode) ;
  finally
    Stack.Free ;
  end ;
end ;

procedure BreadthFirst (Node : TNode ; OnProcessNode : TNodeEvent) ;
var
  Queue : TQueue ;

begin
  Queue := TQueue.Create ;
  try
    GraphIterator (Queue,Node,OnProcessNode) ;
  finally
    Queue.Free ;
  end ;
end ;

///////////////////////////////////////////////////////////
{ TGraph }
///////////////////////////////////////////////////////////

constructor TGraph.Create;
begin
  FNodes := TObjectList.Create (true) ;
end;

destructor TGraph.Destroy;
begin
  FNodes.Clear ; // Since it is a TObjectList it will fire the Free method of the owned nodes !
  FNodes.Free ;
end;

procedure TGraph.AddNode (Node: TNode);
begin
  if FNodes.IndexOf (Node) <> -1 then begin
    raise EGraphException.Create ('Node already exists for the given graph') ;
  end ;
  FNodes.Add (Node) ;
  Node.FGraph := Self ;
end;

procedure TGraph.DelNode (Node : TNode) ;
var
  i : integer ;
  iIndex : integer ;
  Edge : TEdge ;

begin
  iIndex := FNodes.IndexOf (Node) ;
  if iIndex = -1 then begin
    raise EGraphException.Create ('Node not found') ;
  end ;

  for i := 0 to Node.PrevEdgeCount - 1 do begin
    Edge := Node.PrevEdges [i] ;
    Edge.PrevNode.DelNextEdge (Edge) ;
  end ;

  for i := 0 to Node.NextEdgeCount - 1 do begin
    Edge := Node.NextEdges [i] ;
    Edge.NextNode.DelPrevEdge (Edge) ;
  end ;

  FNodes.Delete (iIndex) ;
end;

function TGraph.GetNode(iIndex: integer) : TNode ;
begin
  result := FNodes [iIndex] as TNode ;
end;

function TGraph.GetNodesCount: integer;
begin
  result := FNodes.Count ;
end;

function TGraph.GetNodeIndex (Node : TNode) : integer ;
begin
  result := FNodes.IndexOf (Node) ;
end ;

///////////////////////////////////////////////////////////
{ TNode }
///////////////////////////////////////////////////////////

constructor TNode.Create;
begin
  { Can't avoid the double freed of edges when using the owned property
  of the TObjectList. So we set it to false !!! }
  FNextEdges := TObjectList.Create (false) ;
  FPrevEdges := TObjectList.Create (false) ;
end;

destructor TNode.Destroy;
begin
  { The Free method of the TList call its Clear method ;-}
  FPrevEdges.Free ;
  FNextEdges.Free ;
end;

procedure TNode.AddPrevEdge (Edge: TEdge);
begin
  AddEdge (FPrevEdges,Edge) ;
end ;

procedure TNode.AddNextEdge (Edge: TEdge);
begin
  AddEdge (FNextEdges,Edge) ;
end ;

procedure TNode.AddEdge (EdgesList : TObjectList ; Edge: TEdge);
begin
  { Verify that the edge belongs to the node }
  if not ((EdgesList = FNextEdges) and (Edge.PrevNode = Self) or
          (EdgesList = FPrevEdges) and (Edge.NextNode = Self)) then begin
    raise EGraphException.Create ('Edge not linked to the given node') ;
  end ;

  { Avoid double insertion }
  if EdgesList.IndexOf (Edge) <> -1 then begin
    raise EGraphException.Create ('Edge already exists for the given node') ;
  end ;
  EdgesList.Add (Edge) ;
end;

procedure TNode.DelPrevEdge (Edge : TEdge);
begin
  DelEdge (FPrevEdges,Edge) ;
end;

procedure TNode.DelNextEdge (Edge : TEdge);
begin
  DelEdge (FNextEdges,Edge) ;
end;

procedure TNode.DelEdge (EdgesList : TObjectList ; Edge : TEdge);
begin
  if EdgesList.Remove (Edge) = -1 then begin
    raise EGraphException.Create ('Edge not found') ;
  end ;
  Edge.Free ;
end;

function TNode.GetConnected : boolean ;
begin
  result := boolean (FPrevEdges.Count + FNextEdges.Count > 0) ;
end ;

function TNode.GetPrevEdgeIndex (Edge : TEdge) : integer ;
begin
  result := FPrevEdges.IndexOf (Edge) ;
end ;

function TNode.GetNextEdgeIndex (Edge : TEdge) : integer ;
begin
  result := FNextEdges.IndexOf (Edge) ;
end ;

function TNode.GetPrevEdge(iIndex: integer): TEdge;
begin
  result := FPrevEdges [iIndex] as TEdge ;
end;

function TNode.GetPrevEdgeCount: integer;
begin
  result := FPrevEdges.Count ;
end;

function TNode.GetNextEdge(iIndex: integer): TEdge;
begin
  result := FNextEdges [iIndex] as TEdge ;
end;

function TNode.GetNextEdgeCount: integer;
begin
  result := FNextEdges.Count ;
end;

///////////////////////////////////////////////////////////
{ TEdge }
///////////////////////////////////////////////////////////

constructor TEdge.Create (PrevNode, NextNode : TNode) ;
begin
  if not Assigned (PrevNode) or not Assigned (NextNode) then begin
    raise EGraphException.Create ('Nodes can''t be nil') ;
  end ;
  if PrevNode.Graph <> NextNode.Graph then begin
    raise EGraphException.Create ('Node''s edge must belong to the same graph') ;
  end ;

  FPrevNode := PrevNode ;
  FNextNode := NextNode ;
  PrevNode.AddNextEdge (Self) ;
  NextNode.AddPrevEdge (Self) ;
end;

end.