{
Description :
This unit implement a singleton instance container for registered classes.
Users can register several classes, each time given an initialiazed instance
Then using Take & return symmetrically, users are guarantee to use a singleton !!!
When finish user should unregister its class and free the returned instance

Since the OO model of delphi does not allowed multiple inheritances and since
interface can't handle this issue, classes does not need to derive from
a TSingletonClass. So every objects can be stored and retrieved !

Author :
Michel Derie (derie@cragx.fgov.be)

Distribution :
SourceWare, FunWare, FreeWare, SmileWare, MirrWare, ArmoirWare ;-), ... etc

Remarks & improvements :
On the design, the extensions or anything else ... ARE welcome !

Compiler :
Delphi 5 SP 1 ... Since it is a base classe it should compile on Delphi 3 & 4 also (I guess)

Legal disclaimer :
No warrenty of any kind ! Not fully tested so bugs can be present ;(

History :
Date        Author   Version   Comment
12/2/2001   MDE      v0.1      Creation & release !-) Thanks to torry's site
}
unit SingletonClass ;

interface

uses
  Classes, SysUtils, Contnrs ;

type
  ESingletonException = class (Exception) ;

  TSingleton = class (TObject)
  private
    FClasses : TClassList ;
    FIntObjects, FExtObjects : TObjectList ;
    function FreeObject (AClass : TClass) : TObject ;
    procedure AssertRegisteredClass (AClass : TClass ; bExpected : boolean) ;
    procedure AssertRegisteredObject (AClass : TClass ; ObjectList : TObjectList) ;
  public
    { Public declarations }
    constructor Create ;
    destructor Destroy ; override ;
    function Take (AClass : TClass) : TObject ;
    procedure Return (AClass : TClass) ;
    procedure Register (AClass : TClass ; AObject : TObject) ;
    function UnRegister (AClass : TClass) : TObject ;
  end ;

var
  Singleton : TSingleton ;

implementation

var
  { Just to ensure the unicity of the TSingleton instance itself ;-}
  bInternal : boolean ;

{ TSingleton }

constructor TSingleton.Create  ;
begin
  if not bInternal then begin
    raise Exception.Create ('Singleton instance can''t be created') ;
  end ;

  FClasses := TClassList.Create ;
  { We don't want that the lists own the objects, since we move object form one to the other :
    procedure take
    ...
    obj := Int.items [idx]
    Int.Delete (idx) ; // if owned is true this will call the destroy method of obj !!!
    Ext.Add (obj) ; }
  FIntObjects := TObjectList.Create (false) ;
  FExtObjects := TObjectList.Create (false) ;
end;

destructor TSingleton.Destroy ;
begin
  if not bInternal then begin
    raise ESingletonException.Create ('Singleton instance can''t be destroyed') ;
  end ;

  if FClasses.Count > 0 then begin
    raise ESingletonException.Create ('At least one class wasn''t unregistered') ;
  end ;

  FExtObjects.Free ;
  FIntObjects.Free ;
  FClasses.Free ;
end;

{
Description : return the instance of the given class and delete it
              from its object list (CheckRegisteredObject must be called before)
}
function TSingleton.FreeObject (AClass : TClass) : TObject ;
var
  iIndex : integer ;

begin
  result := nil ;
  iIndex := FIntObjects.FindInstanceOf (AClass) ;
  if iIndex <> -1 then begin
    result := FIntObjects.Items [iIndex] ;
    FIntObjects.Delete (iIndex) ;
  end
  else begin
    iIndex := FExtObjects.FindInstanceOf (AClass) ;
    if iIndex <> -1 then begin
      result := FExtObjects.Items [iIndex] ;
      FExtObjects.Delete (iIndex) ;
    end ;
  end ;
  if not Assigned (result) then begin
    raise ESingletonException.Create ('Internal error') ;
  end ;
end ;

function TSingleton.Take (AClass : TClass): TObject ;
var
  iIndex : integer ;

begin
  AssertRegisteredClass (AClass,true) ;
  AssertRegisteredObject (AClass,FIntObjects) ;

  iIndex := FIntObjects.FindInstanceOf (AClass,true,0) ;
  result := FIntObjects.Items [iIndex] ;
  FIntObjects.Delete (iIndex) ;
  FExtObjects.Add (result) ;
end;

procedure TSingleton.Return (AClass: TClass);
var
  iIndex : integer ;
  AObject : TObject ;

begin
  AssertRegisteredClass (AClass,true) ;
  AssertRegisteredObject (AClass,FExtObjects) ;

  iIndex := FExtObjects.FindInstanceOf (AClass,true,0) ;
  AObject := FExtObjects.Items [iIndex] ;
  FExtObjects.Delete (iIndex) ;
  FIntObjects.Add (AObject) ;
end;

procedure TSingleton.Register(AClass : TClass ; AObject : TObject) ;
begin
  AssertRegisteredClass (AClass,false) ;
  FClasses.Add (AClass) ;
  FIntObjects.Add (AObject) ;
end;


function TSingleton.UnRegister (AClass: TClass) : TObject ;
begin
  AssertRegisteredClass (AClass,true) ;
  { Remove class reference }
  FClasses.Remove (AClass) ;
  { Remove object reference }
  result := FreeObject (AClass) ;
end;

procedure TSingleton.AssertRegisteredClass (AClass : TClass ; bExpected : boolean) ;
var
  bClassExist : boolean ;

begin
  { Does AClass registered ? }
  bClassExist := boolean (FClasses.IndexOf (AClass) <> -1) ;
  { Is it a good news ? }
  if bClassExist = not bExpected then begin
    if bClassExist then begin
      raise ESingletonException.Create ('Class already registred') ;
    end
    else begin
      raise ESingletonException.Create ('Class not found') ;
    end ;
  end ;
end ;

procedure TSingleton.AssertRegisteredObject (AClass : TClass ; ObjectList : TObjectList) ;
begin
  { Does AObject exist ? }
  if ObjectList.FindInstanceOf (AClass) = -1 then begin
    { Oops ... raise an exception based on the given objectlist }
    if ObjectList = FIntObjects then begin
      raise ESingletonException.Create ('Object can''t be took twice') ;
    end
    else begin
      raise ESingletonException.Create ('Object was''nt took') ;
    end ;
  end ;
end;

initialization
  bInternal := true ;
  Singleton := TSingleton.Create ;
  bInternal := false ;

finalization
  bInternal := true ;
  Singleton.Free ;

end.